/* Public libmudflap declarations -*- C -*- */

#ifndef MF_RUNTIME_H
#define MF_RUNTIME_H

#if 1 /* autoconf HAVE_STDINT_H */
#include <stdint.h>
#endif
#include <sys/types.h>
#if ! 1
typedef unsigned long uintptr_t;
#define HAVE_UINTPTR_T 1
/* Define this here, in case an autoconf application was run
   without CFLAGS=-fmudflap but is being compiled with -fmudflap.  */
#endif


/* Global declarations used by instrumentation. */

struct __mf_cache { uintptr_t low; uintptr_t high; };
extern struct __mf_cache __mf_lookup_cache [];
extern uintptr_t __mf_lc_mask;
extern unsigned char __mf_lc_shift;

/* Multithreading support.  */
#ifdef _MUDFLAPTH
/* extern pthread_mutex_t __mf_biglock; */
#endif

/* Codes to describe the type of access to check: __mf_check arg 3 */

#define __MF_CHECK_READ 0
#define __MF_CHECK_WRITE 1


/* Codes to describe a region of memory being registered: __mf_*register arg 3 */

#define __MF_TYPE_NOACCESS 0
#define __MF_TYPE_HEAP 1
#define __MF_TYPE_HEAP_I 2
#define __MF_TYPE_STACK 3
#define __MF_TYPE_STATIC 4
#define __MF_TYPE_GUESS 5


/* The public mudflap API */

#ifdef __cplusplus
extern "C" {
#endif

extern void __mf_check (void *ptr, size_t sz, int type, const char *location)
       __attribute((nothrow));
extern void __mf_register (void *ptr, size_t sz, int type, const char *name) 
       __attribute((nothrow));
extern void __mf_unregister (void *ptr, size_t sz) __attribute((nothrow));
extern unsigned __mf_watch (void *ptr, size_t sz);
extern unsigned __mf_unwatch (void *ptr, size_t sz);
extern void __mf_report ();
extern int __mf_set_options (const char *opts);


/* Redirect some standard library functions to libmudflap.  These are
   done by simple #define rather than linker wrapping, since only
   instrumented modules are meant to be affected.  */

#ifdef _MUDFLAP  
#pragma redefine_extname memcpy __mfwrap_memcpy 
#pragma redefine_extname memmove __mfwrap_memmove
#pragma redefine_extname memset __mfwrap_memset 
#pragma redefine_extname memcmp __mfwrap_memcmp 
#pragma redefine_extname memchr __mfwrap_memchr 
#pragma redefine_extname memrchr __mfwrap_memrchr
#pragma redefine_extname strcpy __mfwrap_strcpy 
#pragma redefine_extname strncpy __mfwrap_strncpy 
#pragma redefine_extname strcat __mfwrap_strcat 
#pragma redefine_extname strncat __mfwrap_strncat
#pragma redefine_extname strcmp __mfwrap_strcmp 
#pragma redefine_extname strcasecmp __mfwrap_strcasecmp 
#pragma redefine_extname strncmp __mfwrap_strncmp 
#pragma redefine_extname strncasecmp __mfwrap_strncasecmp
#pragma redefine_extname strdup __mfwrap_strdup 
#pragma redefine_extname strndup __mfwrap_strndup 
#pragma redefine_extname strchr __mfwrap_strchr 
#pragma redefine_extname strrchr __mfwrap_strrchr
#pragma redefine_extname strstr __mfwrap_strstr 
#pragma redefine_extname memmem __mfwrap_memmem 
#pragma redefine_extname strlen __mfwrap_strlen 
#pragma redefine_extname strnlen __mfwrap_strnlen
#pragma redefine_extname bzero __mfwrap_bzero 
#pragma redefine_extname bcopy __mfwrap_bcopy 
#pragma redefine_extname bcmp __mfwrap_bcmp 
#pragma redefine_extname index __mfwrap_index 
#pragma redefine_extname rindex __mfwrap_rindex
#pragma redefine_extname asctime __mfwrap_asctime
#pragma redefine_extname ctime __mfwrap_ctime
#pragma redefine_extname gmtime __mfwrap_gmtime
#pragma redefine_extname localtime __mfwrap_localtime

/* Disable glibc macros.  */
#define __NO_STRING_INLINES

#endif /* _MUDFLAP */


#ifdef __cplusplus
}
#endif

#endif /* MF_RUNTIME_H */
